/*
    e2floppy.h


    flexemu, an MC6809 emulator running FLEX
    Copyright (C) 1997  W. Schwotzer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef __e2floppy_h__
#define __e2floppy_h__

#include "misc1.h"

#ifndef __fromflex__

#ifdef _MSC_VER
#define W_OK	(2) /* write permission */
#else
#include <unistd.h>
#endif
#include <stdio.h>
#include <sys/stat.h>
#include "mc6809.h"
#include "wd1793.h"

// possible floppy types:

#define NO_FORMAT	(0)	// unformated
#define FLX_FORMAT	(1)	// format defined for this emulator
#define DSK_FORMAT	(2)	// format used for Ben Williams FLEX emulator
#define NAFS_FORMAT	(3)	// map floppy to a native directory
				// (NAtive File System)

#define ERR_SIZE	(200)
#ifndef SEEK_SET
#define SEEK_SET 0
#endif /* #indef SEEK_SET */

#else
#include "typedefs.h"
#endif /* #ifndef __fromflex__ */

#define MAGIC_NUMBER 0x485c9a33L

/* datastructure describing the header of a FLEX disk image file */

struct s_flex_header {
	DWord magic_number;	/* to identify right format	 */
	Byte write_protect;	/* if != 0 disk image is write-prot*/
	Byte sizecode;		/* 128 * 2 ^ n Bytes per Sector	 */
	Byte sides0;		/* nr. of sides on track 0 	 */
	Byte sectors0;		/* nr. of sect. on track 0 (one side)*/
	Byte sides;		/* nr. of sides on track != 0	 */
	Byte sectors;		/* nr. of sectors on track != 0	 */
	Byte tracks;		/* nr. of tracks total		 */
	Byte dummy1;		/* for future extension		 */
	Byte dummy2, dummy3, dummy4, dummy5; /* and for alignment 4*/
};

#ifndef __fromflex__

struct s_formats {
	SDWord		size;		// byte size of disk
	Word		tracks;		// number of tracks
	Word		sectors;	// number of sectors
	Word		dir_sectors;	// number of directory sectors
};

// structure of FLEX system info sector
struct s_sys_info_sector {
	Byte	unused1[16];
	char	disk_name[8];
	char	disk_ext[3];
	Byte	disk_number[2];
	Byte	fc_start_trk;
	Byte	fc_start_sec;
	Byte	fc_end_trk;
	Byte	fc_end_sec;
	Byte	free[2];
	Byte	month;
	Byte	day;
	Byte	year;
	Byte	last_trk;
	Byte	last_sec;
	Byte	unused2[216];
};

#ifdef NAFS

#define MAX_TRACK	79		// max. nr. of tracks - 1
#define MAX_SECTOR	36		// nr of sect. per track (side 0 and 1)
#define INIT_DIR_SECTORS (20-4)		// initial nr. of directory sectors
#define LINK_TABLE_SIZE ((MAX_TRACK+1) * MAX_SECTOR)
#define INIT_NEW_FILES	4		// initial nr of new files to be
					// managed at a time
#define FREE_CHAIN	-1
#define DIRECTORY	-2
#define SYSTEM  	-3
#define NEW_FILE1	-4

struct s_st {
#ifdef MACH_BYTE_ORDER_MSB_FIRST
	Byte sec;
	Byte trk;
#else
	Byte trk;
	Byte sec;
#endif
};

typedef union {
	Word		sec_trk;	// spec. sector and tracknr. of as sect.
	struct s_st	st;
} t_st;

struct s_link_table {
	t_st		next;		// sector and tracknumber of next sect.
	Byte		record_nr[2];	// FLEX record number
	Word		f_record;	// rel position in file / 252
	SWord		file_id;	// index of file in directory
};

// structure of one FLEX directory entry
struct s_dir_entry {
	char 	filename[8];
	char 	file_ext[3];
	Byte	file_attr;
	Byte	reserved1;
	char 	start_trk;
	char 	start_sec;
	Byte	end_trk;
	Byte	end_sec;
	Byte	records[2];
	Byte	sector_map;
	Byte	reserved2;
	Byte	month;
	Byte	day;
	Byte	year;
};

// structure of one FLEX directory sector
struct s_dir_sector {
	Byte	next_trk;
	Byte	next_sec;
	Byte	record_nr[2];
	Byte	unused[12];
	struct s_dir_entry dir_entry[10];
};

struct s_unused_sector {
	Byte	next_trk;
	Byte	next_sec;
	Byte	unused[254];
};

struct s_new_file {
	char	filename[13];
	t_st	first;
	t_st	next;
	Word	f_record;
	FILE	*fp;
};
#endif // #ifdef NAFS


struct s_floppy {
	Byte		ready;		// ready flag
	Byte		write_protect;	// write protect flag of disk
	Word		offset;		// offset for fileheader
	FILE		*fp;		// file pointer of disk image file
	Word		byte_p_track;	// total bytes per track
	Word		byte_p_track0;  // total bytes on track 0
	Word		byte_p_sector;	// bytes per sector
	Word		max_sector;	// max. nr of sectors per side
	Word		max_sector0;	// max. nr of sectors per side track 0
	Word		max_track;	// max. tracknumber of disk 
	Word		track;		// actual tracknumber
	Word		type;		// type of floppy
	char		*ppath;		// pointer to unix path of drive
#ifdef NAFS
	s_link_table	*pflex_links;	// pointer to link table
	s_sys_info_sector *pflex_sys_info; // pointer to flex info sectors
	s_unused_sector	*pflex_unused;	// pointer to flex unused sector (2)
	s_dir_sector	*pflex_directory;// pointer to flex directory
	s_new_file	*pnew_file;	// pointer to new file table
	Word		dir_sectors;	// nr. of dir sectors in flex_directory
	Word		new_files;	// nr. of new file entries 
	t_st		dir_extend;	//  track/sector of dir. extension sect.
#endif // #ifdef NAFS
};


class E2floppy : public Wd1793 {

//
// internal registers
//
//	status			status register (r)
//	drisel			drive selection (w)
//	selected		selected drive as index into struct floppy

protected:

	Byte			status, drisel, selected;

// interface to operating system:
// a floppy structure for any drive connected
// drive nr. 4 means no drive selected
// pfs always points to the selected floppy

protected:

	struct s_floppy		floppy[5], *pfs;
	char			sector_buffer[256];
	char			*disk_dir;

// Internals

private:

	Mc6809			*cpu;
	Inout			*io;

private:

	virtual Byte		 readByte(Word index);
	virtual void		 writeByte(Word index);
	virtual Byte		 driveReady(void);
	virtual Byte		 writeProtect(void);
	virtual Byte		 recordNotFound(void);
	virtual Byte		 positionDrive(void);
	virtual Byte		 seekError(Byte new_track);
	void			 header_for_own_format(
					s_floppy	*floppy,
					s_flex_header	*pheader,
					Byte		wp);
	void			 header_for_ben_format(
					s_floppy	*floppy,
					s_formats	*pformat,
					Byte		wp);
	void			 header_for_no_format(Word drive_nr);
	char 			*open_mode(char *path);
	Byte			 init_disk_image(struct s_floppy *pfs);
	virtual void		 create_boot_sector(Byte sec_buf[]);
	virtual void		 create_sector2(
					Byte	sec_buf[],
					struct	s_formats *fmt);
	virtual void		 create_sys_info_sector(
					Byte	sec_buf[],
					char	*name,
					struct	s_formats *fmt);
	virtual void		 write_dir_sectors(
					FILE *fp,
					struct	s_formats *fmt);
	virtual void		 write_sectors(
					FILE *fp,
					struct	s_formats *fmt);

#ifdef NAFS
private:
	void			header_for_nafs_format(
					s_floppy	*floppy,
					Byte		wp);
	void			fill_flex_directory(
					struct s_floppy *pfs,
					Byte		dwp);
	void			initialize_flex_sys_info_sectors(
					struct s_floppy	*pfs,
					Word		number);
	Byte			init_nafs_drive(struct s_floppy *pfs);
	void			initialize_flex_unused_sector(
					struct s_floppy *pfs);
	void			initialize_flex_directory(
					struct s_floppy *pfs);
	void			initialize_flex_link_table(
					struct s_floppy *pfs);
	void			initialize_new_file_table(
					struct s_floppy *pfs);
	void			close_new_files(Word index);
	void			mount_nafs(
					struct s_floppy	*pfs,
					Word		index);
	void			umount_nafs(Word index);
	void			free_nafs_memory(Word index);
	Byte			open_files(Word index);
	SWord			next_free_dir_entry(struct s_floppy *pfs);
	char			*unix_filename(SWord dir_index);
	char			*to_unix_filename(s_dir_entry *pde);
	void			scan_unix_filename(char *pfn, char *pfilename);
	Byte			add_to_link_table(
					struct s_floppy	*pfs,
					SWord		dir_index,
					off_t		size,
					Byte		random,
					t_st		*begin,
					t_st		*end);
	void			add_to_directory(
					struct s_floppy *pfs,
					char *name,
					char *ext,
					SWord dir_index,
					Byte random,
					struct stat *pstat,
					t_st *begin,
					t_st *end,
					Byte wp);
	void			modify_random_file(
					char	*path,
					struct	stat *pstat,
					t_st	*pbegin);
	Byte			is_flex_filename(
					char	*filename,
					char	*name,
					char	*ext);
	Word			is_in_file_random(
					const char *ppath,
					const char *pfilename);
	void			check_for_delete(
					SWord		dir_index,
					s_dir_sector	*buffer);
	void			check_for_extend(
					SWord		dir_index,
					s_dir_sector	*buffer);
	void			check_for_rename(
					SWord		dir_index,
					s_dir_sector	*pb);
	Byte			check_for_new_file(
					SWord dir_index,
					s_dir_sector *pd);
	Byte			extend_directory(
					struct s_floppy	*pfs,
					SWord		index,
					s_dir_sector	*pdb);
	SWord			set_file_time(
					char	*ppath,
					Byte	month,
					Byte	day,
					Byte	year);
	t_st			*link_address(void);
	Byte			last_of_free_chain(Byte tr, Byte sr);
	void			read_nafs_sector(void);
	void			write_nafs_sector(void);
	SWord			index_of_new_file(Byte track, Byte sector);
	Word			record_nr_of_new_file(SWord new_file_index,
					Word index);
	void			change_file_id(
					SWord index,
					SWord old_id,
					SWord new_id);
#endif // NAFS

// public interface for I/O-device:

public:

	virtual void		 resetIo(void);
	virtual Byte		 readIo(Word offset);
	virtual void		 writeIo(Word offset, Byte val);

	virtual void		 disk_directory(char *x_disk_dir);
	virtual void		 mount_all_drives(char *drive[]);
	virtual Byte		 update_all_drives(void);
	virtual Byte		 umount_all_drives(void);
	virtual Byte		 mount_drive(char *path, Word drive_nr);
	virtual Byte		 format_disk(Word type, SDWord size,char *name);
	virtual Byte		 update_drive(Word drive_nr);
	virtual Byte		 umount_drive(Word drive_nr);
	virtual char		*drive_info(Word drive_nr);

				 E2floppy(Inout *x_io, Mc6809 *x_cpu);
				~E2floppy(void);
};

#endif /* #ifndef __fromflex__ */
#endif /* __e2floppy_h__ */

